@@ -1,5 +1,6 @@
 #ifndef __keyATM_weightCov__INCLUDED__
 #define __keyATM_weightCov__INCLUDED__
+#define EIGEN_PERMANENTLY_DISABLE_STUPID_WARNINGS
 
 #include <Rcpp.h>
 #include <RcppEigen.h>
@@ -16,19 +17,15 @@ class LDAcov : public LDAbase, public keyATMcov
 {
   public:
     // Constructor
-    LDAcov(List model_, const int iter_) :
-      keyATMmeta(model_, iter_),
-      LDAbase(model_, iter_),
-      keyATMcov(model_, iter_) {};
-
+    LDAcov(List model_) :
+      keyATMmeta(model_),
+      LDAbase(model_),
+      keyATMcov(model_) {};
 
     // Functions
-    void iteration_single(int it) final;
-    double loglik_total() final;
-
+    void iteration_single(int it) override final;
+    double loglik_total() override final;
 };
 
 #endif
 
-
-
