@@ -6,11 +6,10 @@ using namespace std;
 
 # define PI_V   3.14159265358979323846  /* pi */
 
-keyATMmeta::keyATMmeta(List model_, const int iter_)
+keyATMmeta::keyATMmeta(List model_)
 {
   // Constructor
   model = model_;
-  iter = iter_;
 }
 
 
@@ -29,6 +28,15 @@ void keyATMmeta::fit()
 }
 
 
+void keyATMmeta::resume_fit()
+{
+  // Resume from the saved object
+  read_data();
+  resume_initialize();
+  iteration();
+}
+
+
 void keyATMmeta::read_data()
 {
   // `common` reads data required in all models
@@ -146,7 +154,6 @@ void keyATMmeta::initialize_common()
   int doc_len;
   IntegerVector doc_s, doc_z, doc_w;
 
-
   //
   // Construct vocab weights
   //
@@ -184,6 +191,15 @@ void keyATMmeta::initialize_common()
     vocab_weights = VectorXd::Constant(num_vocab, 1.0);
   }
 
+  // Store weights
+  NumericVector vocab_weights_R = stored_values["vocab_weights"];
+
+  for (int v = 0; v < num_vocab; ++v) {
+    vocab_weights_R[v] = vocab_weights(v);
+  }
+  stored_values["vocab_weights"] = vocab_weights_R;
+  model["stored_values"] = stored_values;
+
 
   //
   // Construct data matrices
@@ -219,20 +235,6 @@ void keyATMmeta::initialize_common()
   // Use during the iteration
   z_prob_vec = VectorXd::Zero(num_topics);
 
-  // Use labels to initialize beta (prior for topic-word distributions)
-  // if (model_settings.containsElementNamed("labels")) {
-  //   use_labels = 1;
-  //   initialize_betas();  // do not use labels for beta for now
-  // } else {
-  //   use_labels = 0;
-  //   Vbeta = (double)num_vocab * beta;
-  //
-  //   Lbeta_sk = VectorXd::Zero(num_topics);
-  //   for (int k = 0; k < num_topics; k++) {
-  //     Lbeta_sk(k) = (double)keywords_num[k] * beta_s;
-  //   }
-  // }
-  use_labels = 0;
   Vbeta = (double)num_vocab * beta;
 
   Lbeta_sk = VectorXd::Zero(num_topics);
@@ -280,71 +282,28 @@ void keyATMmeta::weights_normalize_total()
 }
 
 
-void keyATMmeta::initialize_betas()
+//
+// Initializing using the resume data
+//
+void keyATMmeta::resume_initialize()
 {
-  // Initialize betas using the label information
-  IntegerVector label_vec = model_settings["labels"];
-  int label;
-  int v;
-  int doc_len;
-  IntegerVector doc_w;
-
-  // Initialize beta matrix
-  beta_s0kv = MatrixXd::Constant(num_topics, num_vocab, beta);
-  beta_s1kv.resize(num_topics, num_vocab);
-
-  vector<Triplet> trip_beta_s1;
-
-  for (int k = 0; k < keyword_k; ++k) {
-    for (auto &v : keywords[k]) {
-      trip_beta_s1.push_back(Triplet(k, v, beta_s));
-    }
-  }
-
-
-  // Add values based on the observed counts
-  for (int doc_id = 0; doc_id < num_doc; ++doc_id) {
-    label = label_vec[doc_id];
-    if (label < 0)
-      continue;
-
-    doc_w = W[doc_id];
-    doc_len = doc_each_len[doc_id];
-
-    for (int w_pos = 0; w_pos < doc_len; ++w_pos) {
-      v = doc_w[w_pos];
-
-      if (use_weights) {
-        beta_s0kv(label, v) += vocab_weights(v);
-
-        if (keywords[label].find(v) != keywords[label].end()){
-          trip_beta_s1.push_back(Triplet(label, v, vocab_weights(v)));
-        }
-      } else {
-        beta_s0kv(label, v) += 1.0;
-
-        if (keywords[label].find(v) != keywords[label].end()){
-          trip_beta_s1.push_back(Triplet(label, v, vocab_weights(v)));
-        }
-      }
-    }
-  }
-
-  // Make beta_s1kv as a sparse matrix
-  beta_s1kv.setFromTriplets(trip_beta_s1.begin(), trip_beta_s1.end());
-
-  // Pre-Cauculation for speed up
-  Vbeta_k = beta_s0kv.rowwise().sum();
-  Lbeta_sk = beta_s1kv * VectorXd::Ones(beta_s1kv.cols()); // beta_s1kv.rowwise().sum();
+  initialize_common();
+  resume_initialize_specific();
 }
 
 
 void keyATMmeta::iteration()
 {
-  // Iteration
-  Progress progress_bar(iter, !(bool)verbose);
+  // Calculation thte number of iterations
+  iter = options_list["iterations"];  // total number of iterations after this fitting, defined at the class level
+  int iter_new = options_list["iter_new"];  // how many iterations to add
+  int iter_start = iter - iter_new;  // starting iteration number
 
-  for (int it = 0; it < iter; ++it) {
+  // Iteration
+  SEXP progress_bar = PROTECT(cli_progress_bar(iter_new, NULL));
+  cli_progress_set_name(progress_bar, "Fitting the model");
+
+  for (int it = iter_start; it < iter; ++it) {
     // Run iteration
     iteration_single(it);
 
@@ -359,12 +318,18 @@ void keyATMmeta::iteration()
     }
 
     // Progress bar
-    progress_bar.increment();
+    if (CLI_SHOULD_TICK) {
+      cli_progress_set(progress_bar, it - iter_start);
+    }
 
     // Check keybord interruption to cancel the iteration
     checkUserInterrupt();
   }
 
+  // Progress bar
+  cli_progress_done(progress_bar);
+  UNPROTECT(1);
+
   model["model_fit"] = model_fit;
 }
 
@@ -374,7 +339,7 @@ void keyATMmeta::sampling_store(int r_index)
   // Store likelihood and perplexity during the sampling
 
   double loglik;
-  loglik = (use_labels) ? loglik_total_label() : loglik_total();
+  loglik = loglik_total();
   double perplexity = exp(-loglik / (double)total_words_weighted);
 
   NumericVector model_fit_vec;
@@ -458,7 +423,7 @@ int keyATMmeta::sample_z(VectorXd &alpha, int z, int s,
   n_dk(doc_id, z) -= vocab_weights(w);
   n_dk_noWeight(doc_id, z) -= 1.0;
 
-  new_z = -1; // debug
+  new_z = -1; // initialize
   if (s == 0) {
     for (int k = 0; k < num_topics; ++k) {
 
@@ -513,85 +478,6 @@ int keyATMmeta::sample_z(VectorXd &alpha, int z, int s,
 }
 
 
-int keyATMmeta::sample_z_label(VectorXd &alpha, int z, int s,
-                         int w, int doc_id)
-{
-  int new_z;
-  double numerator, denominator;
-  double sum;
-
-  // For labeled data!
-  // remove data
-  if (s == 0) {
-    n_s0_kv(z, w) -= vocab_weights(w);
-    n_s0_k(z) -= vocab_weights(w);
-  } else if (s == 1) {
-    n_s1_kv.coeffRef(z, w) -= vocab_weights(w);
-    n_s1_k(z) -= vocab_weights(w);
-  } else {
-    Rcerr << "Error at sample_z, remove" << std::endl;
-  }
-
-  n_dk(doc_id, z) -= vocab_weights(w);
-  n_dk_noWeight(doc_id, z) -= 1.0;
-
-  new_z = -1; // debug
-  if (s == 0) {
-    for (int k = 0; k < num_topics; ++k) {
-
-      numerator = (beta_s0kv(k, w) + n_s0_kv(k, w)) *
-        (n_s0_k(k) + prior_gamma(k, 1)) *
-        (n_dk(doc_id, k) + alpha(k));
-
-      denominator = (Vbeta_k(k) + n_s0_k(k)) *
-        (n_s1_k(k) + prior_gamma(k, 0) + n_s0_k(k) + prior_gamma(k, 1));
-
-      z_prob_vec(k) = numerator / denominator;
-    }
-
-    sum = z_prob_vec.sum(); // normalize
-    new_z = sampler::rcat_without_normalize(z_prob_vec, sum, num_topics); // take a sample
-
-  } else {
-    for (int k = 0; k < num_topics; ++k) {
-      if (keywords[k].find(w) == keywords[k].end()) {
-        z_prob_vec(k) = 0.0;
-        continue;
-      } else {
-        numerator = (beta_s1kv.coeffRef(k, w) + n_s1_kv.coeffRef(k, w)) *
-          (n_s1_k(k) + prior_gamma(k, 0)) *
-          (n_dk(doc_id, k) + alpha(k));
-        denominator = (Lbeta_sk(k) + n_s1_k(k) ) *
-          (n_s1_k(k) + prior_gamma(k, 0) + n_s0_k(k) + prior_gamma(k, 1));
-
-        z_prob_vec(k) = numerator / denominator;
-      }
-    }
-
-
-    sum = z_prob_vec.sum();
-    new_z = sampler::rcat_without_normalize(z_prob_vec, sum, num_topics); // take a sample
-
-  }
-
-  // add back data counts
-  if (s == 0) {
-    n_s0_kv(new_z, w) += vocab_weights(w);
-    n_s0_k(new_z) += vocab_weights(w);
-  } else if (s == 1) {
-    n_s1_kv.coeffRef(new_z, w) += vocab_weights(w);
-    n_s1_k(new_z) += vocab_weights(w);
-  } else {
-    Rcerr << "Error at sample_z, add" << std::endl;
-  }
-  n_dk(doc_id, new_z) += vocab_weights(w);
-  n_dk_noWeight(doc_id, new_z) += 1.0;
-
-  return new_z;
-}
-
-
-
 int keyATMmeta::sample_s(int z, int s, int w, int doc_id)
 {
   int new_s;
@@ -642,65 +528,6 @@ int keyATMmeta::sample_s(int z, int s, int w, int doc_id)
 }
 
 
-int keyATMmeta::sample_s_label(VectorXd &alpha, int z, int s,
-                 int w, int doc_id)
-{
-  int new_s;
-  double numerator, denominator;
-  double s0_prob;
-  double s1_prob;
-  double sum;
-
-  // For labeled data!
-  // remove data
-  if (s == 0) {
-    n_s0_kv(z, w) -= vocab_weights(w);
-    n_s0_k(z) -= vocab_weights(w);
-  } else {
-    n_s1_kv.coeffRef(z, w) -= vocab_weights(w);
-    n_s1_k(z) -= vocab_weights(w);
-  }
-
-  // newprob_s1()
-
-  numerator = (beta_s1kv.coeffRef(z, w) + n_s1_kv.coeffRef(z, w)) *
-    ( n_s1_k(z) + prior_gamma(z, 0) );
-  denominator = (Lbeta_sk(z) + n_s1_k(z) );
-  s1_prob = numerator / denominator;
-
-  // newprob_s0()
-  numerator = (beta_s0kv(z, w) + n_s0_kv(z, w)) *
-    (n_s0_k(z) + prior_gamma(z, 1));
-
-  denominator = (Vbeta_k(z) + n_s0_k(z) );
-  s0_prob = numerator / denominator;
-
-  // Normalize
-  sum = s0_prob + s1_prob;
-
-  s1_prob = s1_prob / sum;
-  new_s = R::runif(0,1) <= s1_prob;  //new_s = Bern(s0_prob, s1_prob);
-
-  // add back data counts
-  if (new_s == 0) {
-    n_s0_kv(z, w) += vocab_weights(w);
-    n_s0_k(z) += vocab_weights(w);
-  } else {
-    n_s1_kv.coeffRef(z, w) += vocab_weights(w);
-    n_s1_k(z) += vocab_weights(w);
-  }
-
-  return new_s;
-}
-
-
-double keyATMmeta::loglik_total_label()
-{
-  // Should be defined in each model
-  return 0.0;
-}
-
-
 // Utilities
 //
 double keyATMmeta::gammapdfln(const double x, const double a, const double b)
@@ -757,27 +584,5 @@ double keyATMmeta::gammaln_frac(const double value, const int count)
 List keyATMmeta::return_model()
 {
   // Return output to R
-
-  if (use_labels) {
-    // Return prior to use in R
-    NumericMatrix R_betas0 = Rcpp::wrap(beta_s0kv);
-    SEXP R_betas1 = Rcpp::wrap(beta_s1kv);
-
-    priors_list.push_back(R_betas0, "beta_s0");
-    priors_list.push_back(R_betas1, "beta_s1");
-    model["priors"] = priors_list;
-  }
-
-  // Vocabulary weights
-  NumericVector vocab_weights_R = stored_values["vocab_weights"];
-
-  for (int v = 0; v < num_vocab; v++) {
-    vocab_weights_R[v] = vocab_weights(v);
-  }
-  stored_values["vocab_weights"] = vocab_weights_R;
-  model["stored_values"] = stored_values;
-
-  model["stored_values"] = stored_values;
-
   return model;
 }
